<?php

namespace FondOfSpryker\Zed\Contentful\Business;

use Contentful\Delivery\Client;
use FondOfSpryker\Shared\Contentful\KeyBuilder\EntryKeyBuilder;
use FondOfSpryker\Shared\Contentful\KeyBuilder\IdentifierKeyBuilder;
use FondOfSpryker\Zed\Contentful\Business\Client\ContentfulAPIClient;
use FondOfSpryker\Zed\Contentful\Business\Client\ContentfulAPIClientInterface;
use FondOfSpryker\Zed\Contentful\Business\Client\ContentfulMapper;
use FondOfSpryker\Zed\Contentful\Business\Client\ContentfulMapperInterface;
use FondOfSpryker\Zed\Contentful\Business\Importer\Importer;
use FondOfSpryker\Zed\Contentful\Business\Importer\ImporterInterface;
use FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\EntryStorageImporterPlugin;
use FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\IdentifierStorageImporterPlugin;
use FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\ImporterPluginInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Asset\AssetFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Boolean\BooleanFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Collection\CollectionFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Entry\EntryMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Entry\EntryMapperInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\CustomFieldMapperCollection;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\CustomFieldMapperCollectionInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\DefaultFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\FieldMapperLocator;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\FieldMapperLocatorInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperCollection;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperCollectionInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface;
use FondOfSpryker\Zed\Contentful\Business\Storage\Link\LinkFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Object\ObjectFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Reference\ReferenceFieldMapper;
use FondOfSpryker\Zed\Contentful\Business\Storage\Text\TextFieldMapper;
use FondOfSpryker\Zed\Contentful\ContentfulDependencyProvider;
use Spryker\Client\Storage\StorageClientInterface;
use Spryker\Shared\KeyBuilder\KeyBuilderInterface;
use Spryker\Zed\Kernel\Business\AbstractBusinessFactory;
use Spryker\Zed\Locale\Business\LocaleFacadeInterface;

/**
 * @method \FondOfSpryker\Zed\Contentful\ContentfulConfig getConfig()
 */
class ContentfulBusinessFactory extends AbstractBusinessFactory
{
    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Importer\ImporterInterface
     */
    public function createImporter(): ImporterInterface
    {
        return new Importer(
            $this->createContentfulAPIClient(),
            $this->createContentfulMapper(),
            $this->createEntryMapper(),
            $this->getImporterPlugins(),
            $this->getConfig()->getLocaleMapping()
        );
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\ImporterPluginInterface[]
     */
    protected function getImporterPlugins(): array
    {
        return [
            $this->createEntryStorageImporterPlugin(),
            $this->createIdentifierImporterPlugin(),
        ];
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\ImporterPluginInterface
     */
    protected function createEntryStorageImporterPlugin(): ImporterPluginInterface
    {
        return new EntryStorageImporterPlugin(
            $this->createEntryKeyBuilder(),
            $this->getStorageClient(),
            $this->getConfig()->getFieldNameActive()
        );
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Importer\Plugin\ImporterPluginInterface
     */
    protected function createIdentifierImporterPlugin(): ImporterPluginInterface
    {
        return new IdentifierStorageImporterPlugin(
            $this->createIdentifierKeyBuilder(),
            $this->getStorageClient(),
            $this->getConfig()->getFieldNameActive(),
            $this->getConfig()->getFieldNameIdentifier()
        );
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Entry\EntryMapperInterface
     */
    protected function createEntryMapper(): EntryMapperInterface
    {
        return new EntryMapper($this->createFieldMapperLocator());
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\FieldMapperLocatorInterface
     */
    public function createFieldMapperLocator(): FieldMapperLocatorInterface
    {
        return new FieldMapperLocator(
            $this->createDefaultFieldMapper(),
            $this->createTypeFieldMapperCollection(),
            $this->createCustomFieldMapperCollection()
        );
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\CustomFieldMapperCollectionInterface
     */
    public function createCustomFieldMapperCollection(): CustomFieldMapperCollectionInterface
    {
        return new CustomFieldMapperCollection();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperCollectionInterface
     */
    public function createTypeFieldMapperCollection(): TypeFieldMapperCollectionInterface
    {
        $collection = new TypeFieldMapperCollection();
        $collection->add($this->createAssetFieldMapper());
        $collection->add($this->createBooleanFieldMapper());
        $collection->add($this->createCollectionFieldMapper());
        $collection->add($this->createReferenceFieldMapper());
        $collection->add($this->createLinkFieldMapper());
        $collection->add($this->createTextFieldMapper());
        $collection->add($this->createObjectFieldMapper());
        return $collection;
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createDefaultFieldMapper(): TypeFieldMapperInterface
    {
        return new DefaultFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createTextFieldMapper(): TypeFieldMapperInterface
    {
        return new TextFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createAssetFieldMapper(): TypeFieldMapperInterface
    {
        return new AssetFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createBooleanFieldMapper(): TypeFieldMapperInterface
    {
        return new BooleanFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createCollectionFieldMapper(): TypeFieldMapperInterface
    {
        return new CollectionFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createReferenceFieldMapper(): TypeFieldMapperInterface
    {
        return new ReferenceFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createLinkFieldMapper(): TypeFieldMapperInterface
    {
        return new LinkFieldMapper();
    }

    /**
     * @return \Spryker\Shared\KeyBuilder\KeyBuilderInterface
     */
    protected function createEntryKeyBuilder(): KeyBuilderInterface
    {
        return new EntryKeyBuilder();
    }

    /**
     * @return \Spryker\Shared\KeyBuilder\KeyBuilderInterface
     */
    protected function createIdentifierKeyBuilder(): KeyBuilderInterface
    {
        return new IdentifierKeyBuilder();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Storage\Field\TypeFieldMapperInterface
     */
    protected function createObjectFieldMapper(): TypeFieldMapperInterface
    {
        return new ObjectFieldMapper();
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Client\ContentfulAPIClientInterface
     */
    protected function createContentfulAPIClient(): ContentfulAPIClientInterface
    {
        return new ContentfulAPIClient($this->createContentfulClient());
    }

    /**
     * @return \FondOfSpryker\Zed\Contentful\Business\Client\ContentfulMapperInterface
     */
    protected function createContentfulMapper(): ContentfulMapperInterface
    {
        return new ContentfulMapper($this->getConfig()->getDefaultLocale());
    }

    /**
     * @return \Contentful\Delivery\Client
     */
    protected function createContentfulClient(): Client
    {
        return new Client(
            $this->getConfig()->getAccessToken(),
            $this->getConfig()->getSpaceId(),
            'master',
            false,
            $this->getConfig()->getDefaultLocale()
        );
    }

    /**
     * @return \Spryker\Client\Storage\StorageClientInterface
     */
    protected function getStorageClient(): StorageClientInterface
    {
        return $this->getProvidedDependency(ContentfulDependencyProvider::STORAGE_CLIENT);
    }

    /**
     * @return \Spryker\Zed\Locale\Business\LocaleFacadeInterface
     */
    protected function getLocaleFacade(): LocaleFacadeInterface
    {
        return $this->getProvidedDependency(ContentfulDependencyProvider::LOCALE_FACADE);
    }
}
